#ifndef Elliptic_Mt2_332_Calculator_H
#define Elliptic_Mt2_332_Calculator_H

#include "Mt2/Mt2_332_Calculator.h"
#include "Minuit2/FCNBase.h"
#include <vector>

/** Adapted from http://daneel.physics.ucdavis.edu/~zhenyuhan/mt2.html
    which is described in 
    Minimal Kinematic Constraints and MT2
    arXiv:0810.5178
    Hsin-Chia Cheng, Zhenyu Han 
*/

namespace Mt2 {

  class Elliptic_Mt2_332_Calculator : public Mt2_332_Calculator {
  public:
    double mt2_332(const LorentzTransverseVector& visibleA, // 3 d.o.f. 
		   const LorentzTransverseVector& visibleB, // 3 d.o.f.
		   const TwoVector& ptmiss,                 // 2 d.o.f.
		   const double mInvisible);
    double mt2_332_Sq(const LorentzTransverseVector& visibleA, // 3 d.o.f. 
		      const LorentzTransverseVector& visibleB, // 3 d.o.f.
		      const TwoVector& ptmiss,                 // 2 d.o.f.
		      const double mInvisible);
    Elliptic_Mt2_332_Calculator();
    
  private:
    void   mt2_bisect();
    void   mt2_massless();
    void   set_momenta(double *pa0, double *pb0, double* pmiss0);
    void   set_mn(double mn);
    double get_mt2();
    void   print();
    int    nevt;
    bool   solved;

    // should be modifiable...
    double SCANSTEP;
    double MIN_MASS;
    double PRECISION;

    double mt2_b;
    int needscan;
    
    int    nsols(double Dsq);
    int    nsols_massless(double Dsq);
    inline int    signchange_n( long double t1, long double t2, long double t3, long double t4, long double t5);
    inline int    signchange_p( long double t1, long double t2, long double t3, long double t4, long double t5);
    int scan_high(double &Deltasq_high);
    //data members
    double pax, pay, ma, Ea;
    double pmissx, pmissy;
    double pbx, pby, mb, Eb;
    double mn;
    
    //auxiliary definitions
    double masq, Easq;
    double mbsq, Ebsq;
    double pmissxsq, pmissysq;
    double mnsq;
    
    //auxiliary coefficients
    double a1, b1, c1, a2, b2, c2, d1, e1, f1, d2, e2, f2;
    double d11, e11, f12, f10, d21, d20, e21, e20, f22, f21, f20;
  };

} //end namespace Mt2

#endif
