#include "Mt2/Mt2Calculator.h"
#include "Mt2/Mt2Vectors.h"

extern "C" {
  double restricted_mt2__(double*,double*,double*,double*,int*);
  double strminuit4_(double*,double*,double*,double*,double*,double*,double*);
  double strminuit2_(double*,double*,double*,double*,double*,double*,double*);
}

namespace Mt2{

double mt2Summers(const TwoVector& vis1, 
		  const TwoVector& vis2, 
		  const TwoVector& missing, 
		  double mlsp){
  double v1[2];
  v1[0]=vis1.px();
  v1[1]=vis1.py();
  double v2[2];
  v2[0]=vis2.px();
  v2[1]=vis2.py();
  double vm[2];
  vm[0]=missing.px();
  vm[1]=missing.py();
  double m[1];
  m[0]=mlsp;
  int debug[0];
  debug[0]=0;
  return restricted_mt2__(v1, v2, vm, m, debug);
}

double mt2Lester4(const LorentzTransverseVector& vis1, 
		  const LorentzTransverseVector& vis2, 
		  const TwoVector& missing, 
		  double mlsp){
  double px[2];
  px[0]=vis1.px();
  px[1]=vis2.px();

  double py[2];
  py[0]=vis1.py();
  py[1]=vis2.py();

  double mlep[2];
  mlep[0]=vis1.mass();
  mlep[1]=vis2.mass();

  double pxvis[1];
  pxvis[0]=-missing.px();

  double pyvis[1];
  pyvis[0]=-missing.py();

  TwoVector other = - missing - vis1.transverse() - vis2.transverse();
  
  double etvis[1];
  etvis[0]= vis1.Et() + vis2.Et() + other.pt();
  
  double chi[1];
  chi[0]=mlsp;
  
  int debug[0];
  debug[0]=1;
  return strminuit4_(px, py, etvis, pxvis, pyvis, mlep, chi);
}

double mt2Lester3(const TwoVector& vis1,
                  const TwoVector& vis2,
                  const double massEachVis,
                  const TwoVector& missing,
                  double mlsp){ 
  double px[2];
  px[0]=vis1.px();
  px[1]=vis2.px();

  double py[2];
  py[0]=vis1.py();
  py[1]=vis2.py();

  double mlep[2];
  mlep[0]=massEachVis;
  mlep[1]=massEachVis;

  double pxvis[1];
  pxvis[0]=-missing.px();

  double pyvis[1];
  pyvis[0]=-missing.py();

  TwoVector other = - missing - vis1 - vis2;
  
  double etvis[1];
  etvis[0] = sqrt(massEachVis*massEachVis + vis1.ptsq()) 
           + sqrt(massEachVis*massEachVis + vis2.ptsq()) 
           + other.pt();
  
  double chi[1];
  chi[0]=mlsp;
  
  int debug[0];
  debug[0]=1;
  return strminuit4_(px, py, etvis, pxvis, pyvis, mlep, chi);
}

double mt2Lester2(const LorentzTransverseVector& vis1, 
		  const LorentzTransverseVector& vis2, 
		  const TwoVector& missing, 
		  double mlsp){
  double px[2];
  px[0]=vis1.px();
  px[1]=vis2.px();

  double py[2];
  py[0]=vis1.py();
  py[1]=vis2.py();
  
  double pz[2];
  pz[0]=0.;
  pz[1]=0.;
  
  double mlep[2];
  mlep[0]=vis1.mass();
  mlep[1]=vis2.mass();

  double pxmiss[1];
  pxmiss[0]=missing.px();

  double pymiss[1];
  pymiss[0]=missing.py();

  double chi[1];
  chi[0]=mlsp;
  
  int debug[0];
  debug[0]=1;
  return strminuit2_(px, py, pz, pxmiss, pymiss, mlep, chi);
}

}
