C     Calculates the variable M_T2 under the conditions that 
C     the masses of the 2 visible (pseudo-)particles can be neglected.
C
C     Please cite 
C     Phys.Lett.B463:99-103,1999 C.G. Lester, D.J. Summers and 
C     J.Phys.G29:2343-2363,2003 Alan Barr, Christopher Lester and Phil Stephens
C
      double precision function 
     &     restricted_mt2(pl1_in,pl2_in,pm_in,mlsp_in,debug)
      
      implicit none
      double precision mt2
      double precision pl1(2),pl2(2),pm(2),mlsp,s
      double precision pl1_in(2),pl2_in(2),pm_in(2),mlsp_in
      common/vars/pl1,pl2,pm,mlsp
      double precision m,dmt2dm
      double precision zeroin,mt2m,top,a,a2,bot,smin
      parameter(a=1.d-8)
      parameter(a2=1.d-4)
      double precision gradient
      external gradient
      integer debug
      integer i

      do i=1,2
         pl1(i)=pl1_in(i)
         pl2(i)=pl2_in(i)
         pm(i)=pm_in(i)
      enddo
      mlsp=mlsp_in

      if (debug.gt.0) then
         write(*,*) 'input : p1t  : ',pl1 
         write(*,*) 'input : p2t  : ',pl2 
         write(*,*) 'input : ptm  : ',pm
         write(*,*) 'input : mlsp : ',mlsp 
      endif

C     The parameter below gives the maximum invariant mass of the 
C     whole system, and is used in the initial setup only
      s=1.96D8   ! 14 TeV squared

      smin=(dsqrt(pl1(1)**2+pl1(2)**2)
     &     +dsqrt(pl2(1)**2+pl2(2)**2)
     &     +dsqrt(pm(1)**2+pm(2)**2+4.d0*mlsp**2))**2
     &     -(pl1(1)+pl2(1)+pm(1))**2
     &     -(pl1(2)+pl2(2)+pm(2))**2
      
      if (s.lt.smin) then
         write(6,*) 'warning,',
     &        ' s is smaller than the minimum allowed value'
         mt2=mlsp**2+2.d0*mt2m(pl1,pl2,pm,mlsp,2.d0*mlsp,dmt2dm)
         restricted_mt2=dsqrt(mt2)
         return
      endif

      bot=max(2.d0*mlsp,a2)
      top=dsqrt(
     &     (dsqrt(s
     &     +(pl1(1)+pl2(1)+pm(1))**2
     &     +(pl1(2)+pl2(2)+pm(2))**2)
     &     -dsqrt(pl1(1)**2+pl1(2)**2)
     &     -dsqrt(pl2(1)**2+pl2(2)**2))**2
     &     -(pm(1)**2+pm(2)**2))
      
      mt2=mt2m(pl1,pl2,pm,mlsp,bot,dmt2dm)
      
      if (dmt2dm.gt.0.d0) then
         mt2=mlsp**2+2.d0*mt2
         restricted_mt2=dsqrt(mt2)
         return
      endif
      
      mt2=mt2m(pl1,pl2,pm,mlsp,top,dmt2dm)
      
      if (dmt2dm.lt.0.d0) then
         mt2=mlsp**2+2.d0*mt2
         restricted_mt2=dsqrt(mt2)
         return
      endif
      
      m=zeroin(bot,top,gradient,a2)
      mt2=mlsp**2+2.d0*mt2m(pl1,pl2,pm,mlsp,m,dmt2dm)
      restricted_mt2=dsqrt(mt2)
      
      return
      end

      function mt2m(pl1,pl2,pm,mlsp,m,dmt2dm)
      implicit none
      double precision mt2m
      double precision pl1(2),pl2(2),pm(2),mlsp
      double precision e1,e2,em,m
      double precision a,b,c,d,th,th1,th2,ph,coseta
      double precision p1p2,pmp1,pmp2,tmp
      double precision dmt2dm,dthdm,dphdm,dddm,dcosetadm
      double precision dadm,dbdm,dcdm,dpmp1dm,dpmp2dm,demdm
      double precision pi
      parameter (pi=3.14159265358979323d0)

      e1=dsqrt(pl1(1)**2+pl1(2)**2)
      e2=dsqrt(pl2(1)**2+pl2(2)**2)
      em=dsqrt(m**2+pm(1)**2+pm(2)**2)

      demdm=m/em

      pmp1=em*e1-pm(1)*pl1(1)-pm(2)*pl1(2)
      pmp2=em*e2-pm(1)*pl2(1)-pm(2)*pl2(2)
      p1p2=e1*e2-pl1(1)*pl2(1)-pl1(2)*pl2(2)

      dpmp1dm=e1*demdm
      dpmp2dm=e2*demdm

c arrange so pmp1 is bigger than pmp2

      if (pmp1.lt.pmp2) then
         tmp=pmp1
         pmp1=pmp2
         pmp2=tmp
         tmp=dpmp1dm
         dpmp1dm=dpmp2dm
         dpmp2dm=tmp
      endif

      a=0.5d0*pmp1
      b=0.5d0*pmp2
      if (mlsp.eq.0.d0) then
         c=1.d0
         dcdm=0.d0
      else
         c=dsqrt(1-4.d0*(mlsp/m)**2)
         if (c.ne.0.d0) then
            dcdm=4.d0*mlsp**2/m**3/c
         else
            dcdm=4.d0*mlsp**2/m**3/1.d-20
         endif
      endif

      coseta=(1.d0-m**2*p1p2/pmp1/pmp2)

      dadm=0.5d0*dpmp1dm
      dbdm=0.5d0*dpmp2dm

      if (pmp2.eq.0.d0) then
         dcosetadm=1.d20
      else
         dcosetadm=-2*m*p1p2/pmp1/pmp2
     &     +m**2*p1p2/(pmp1*pmp2)**2*(dpmp1dm*pmp2+pmp1*dpmp2dm)
      endif

c if the minimum of the p1 curve is above the p2 curve at that
c point then that is the solution
      if ((a*(1.d0-c)).gt.(b*(1.d0+coseta*c))) then
         mt2m=a*(1.d0-c)
         dmt2dm=dadm*(1.d0-c)-a*dcdm
         return
      endif

      d=sqrt(b*b+a*a+2.d0*a*b*coseta)

      dddm=(b*dbdm+a*dadm
     &     +dadm*b*coseta+a*dbdm*coseta+a*b*dcosetadm)/d

      ph=dasin(1.d0/dsqrt(1.d0+b*b*(1-coseta**2)/(a+b*coseta)**2))

      dphdm=
     &     (b*b*(1.d0-coseta**2)*
     &       (dadm+dbdm*coseta+b*dcosetadm)/(a+b*coseta)**3
     &     +(b*b*coseta*dcosetadm-b*dbdm*(1.d0-coseta**2))/
     &     (a+b*coseta)**2)
     &     *dsin(ph)**3/dcos(ph)

      th1=dasin((a-b)/c/d)-ph
      th2=pi-dasin((a-b)/c/d)-ph
      
      if (dcos(th1).gt.dcos(th2)) then
         th=th1
      else
         th=th2
      endif

      dthdm=((dadm-dbdm)/c/d-(a-b)/c/c/d/d*(c*dddm+dcdm*d))
     &     /dcos(th+ph) - dphdm

      mt2m=a*(1.d0-c*dcos(th))
      dmt2dm=dadm*(1.d0-c*dcos(th))
     &     -a*dcdm*dcos(th)
     &     +a*c*dsin(th)*dthdm

      end

      function gradient(m)
      implicit none
      double precision gradient,m,tmp,mt2m
      double precision pl1(2),pl2(2),pm(2),mlsp
      common/vars/pl1,pl2,pm,mlsp

      tmp=mt2m(pl1,pl2,pm,mlsp,m,gradient)

      return
      end

c  To get d1mach, mail netlib
c       send d1mach from core
      double precision function zeroin(ax,bx,f,tol)
      double precision ax,bx,f,tol
c
c      a zero of the function  f(x)  is computed in the interval ax,bx .
c
c  input..
c
c  ax     left endpoint of initial interval
c  bx     right endpoint of initial interval
c  f      function subprogram which evaluates f(x) for any x in
c         the interval  ax,bx
c  tol    desired length of the interval of uncertainty of the
c         final result (.ge.0.)
c
c  output..
c
c  zeroin abscissa approximating a zero of  f  in the interval ax,bx
c
c      it is assumed  that   f(ax)   and   f(bx)   have  opposite  signs
c  this is checked, and an error message is printed if this is not
c  satisfied.   zeroin  returns a zero  x  in the given interval
c  ax,bx  to within a tolerance  4*macheps*abs(x)+tol, where macheps  is
c  the  relative machine precision defined as the smallest representable
c  number such that  1.+macheps .gt. 1.
c      this function subprogram is a slightly  modified  translation  of
c  the algol 60 procedure  zero  given in  richard brent, algorithms for
c  minimization without derivatives, prentice-hall, inc. (1973).
c
      double precision  a,b,c,d,e,eps,fa,fb,fc,tol1,xm,p,q,r,s
      double precision  dabs
c      double precision  d1mach
c   10 eps = d1mach(4)
c set machine accuracy to zero (as I don't have it) and just use tol
   10 eps = 0.d0
      tol1 = eps+1.0d0
c
      a=ax
      b=bx
      fa=f(a)
      fb=f(b)
c     check that f(ax) and f(bx) have different signs
      if (fa .eq.0.0d0 .or. fb .eq. 0.0d0) go to 20
      if (fa * (fb/dabs(fb)) .le. 0.0d0) go to 20
         write(6,2500)
2500     format(1x,'f(ax) and f(bx) do not have different signs,',
     1             ' zeroin is aborting')
         stop
   20 c=a
      fc=fa
      d=b-a
      e=d
   30 if (dabs(fc).ge.dabs(fb)) go to 40
      a=b
      b=c
      c=a
      fa=fb
      fb=fc
      fc=fa
   40 tol1=2.0d0*eps*dabs(b)+0.5d0*tol
      xm = 0.5d0*(c-b)
      if ((dabs(xm).le.tol1).or.(fb.eq.0.0d0)) go to 150
c
c see if a bisection is forced
c
      if ((dabs(e).ge.tol1).and.(dabs(fa).gt.dabs(fb))) go to 50
      d=xm
      e=d
      go to 110
   50 s=fb/fa
      if (a.ne.c) go to 60
c
c linear interpolation
c
      p=2.0d0*xm*s
      q=1.0d0-s
      go to 70
c
c inverse quadratic interpolation
c
   60 q=fa/fc
      r=fb/fc
      p=s*(2.0d0*xm*q*(q-r)-(b-a)*(r-1.0d0))
      q=(q-1.0d0)*(r-1.0d0)*(s-1.0d0)
   70 if (p.le.0.0d0) go to 80
      q=-q
      go to 90
   80 p=-p
   90 s=e
      e=d
      if (((2.0d0*p).ge.(3.0d0*xm*q-dabs(tol1*q))).or.(p.ge.
     *dabs(0.5d0*s*q))) go to 100
      d=p/q
      go to 110
  100 d=xm
      e=d
  110 a=b
      fa=fb
      if (dabs(d).le.tol1) go to 120
      b=b+d
      go to 140
  120 if (xm.le.0.0d0) go to 130
      b=b+tol1
      go to 140
  130 b=b-tol1
  140 fb=f(b)
      if ((fb*(fc/dabs(fc))).gt.0.0d0) go to 20
      go to 30
  150 zeroin=b
      return
      end
