c-----------------------------------------------------------------
      function StrMinuit4(
     &     kkpxlep,
     &     kkpylep,
     &     kketvis,
     &     kkpxvis,
     &     kkpyvis,
     &     kkmlepton,
     &     kkmNtlino)
c-----------------------------------------------------------------
c     This is the single parameter roots minimisation method
c     for calculating MT2 analogous to the TruthBehindTheGlamour
c     equation (25) method, but extended to be able to deal
c     with visible particles of DISSIMILAR masses.  Never written
c     up in a published article, but initially documented in
c     Lester Home Lab Book (near centre) and then re-documented in
c     Lester Lab Book 6 on 4th and 5th July 2007.
c     This method is implemented identically (but with fewer
c     comments and less debug output) in the separate file 
c     str_minuit4.f ... which is also copied to str_minuit5.f with
c     a name change for no apparent reason.
c-----------------------------------------------------------------
      implicit none
c-----Defn
      double precision StrMinuit4
c-----Input Variables
      double precision kkpxlep(2)
      double precision kkpylep(2)
      double precision kketvis
      double precision kkpxvis
      double precision kkpyvis
      double precision kkmlepton(2)
      double precision kkmNtlino
      double precision n1aSq,n1bSq,n2aSq,n2bSq,
     &                 p2ax,p2ay,p2bx,p2by,p2aSq,p2bSq
c-----Includes
      include 'MINI4.INC'

      call FillMini4BlockWith(
     &     kkpxlep,
     &     kkpylep,
     &     kketvis,
     &     kkpxvis,
     &     kkpyvis,
     &     kkmlepton,
     &     kkmNtlino)

      p2ax=-pxvis-mNtlino*pxlep(1)/mlepton(1)
      p2ay=-pyvis-mNtlino*pylep(1)/mlepton(1)
      p2bx=-pxvis-mNtlino*pxlep(2)/mlepton(2)
      p2by=-pyvis-mNtlino*pylep(2)/mlepton(2)

      p2aSq=p2ax**2+p2ay**2
      p2bSq=p2bx**2+p2by**2

      n1aSq=(mlepton(1)+mNtlino)**2
      n1bSq=(mlepton(2)+mNtlino)**2

      n2aSq=(mlepton(2))**2+mNtlino**2+2.0D0*(
     &             etlep(2)*sqrt(mNtlino**2+p2aSq)
     &            -pxlep(2)*p2ax
     &            -pylep(2)*p2ay
     &           )
      n2bSq=(mlepton(1))**2+mNtlino**2+2.0D0*(
     &             etlep(1)*sqrt(mNtlino**2+p2bSq)
     &            -pxlep(1)*p2bx
     &            -pylep(1)*p2by
     &           )

C-----First account for non-crossing point solutions!

      if (n1aSq.ge.n2aSq) then
c---------Real answer occurs when a's ntlino is at rest wrt a.
         StrMinuit4=sqrt(abs(n1aSq))
c         print *,'4RESETTINGA ',StrMinuit4
         return
      endif
      if (n1bSq.ge.n2bSq) then
c---------Real answer occurs when b's ntlino is at rest wrt b.
         StrMinuit4=sqrt(abs(n1bsq))
c         print *,'4RESETTINGB ',StrMinuit4
         return
      endif
c      if (n1aSq.lt.n2aSq.and.n1bSq.lt.n2bSq) then
c         print *,'4NOT'
c      endif
      if (n1aSq.ge.n2aSq.and.n1bSq.ge.n2bSq
     &          .and.(.not.(sqrt(abs(n1aSq)).eq.sqrt(abs(n1bsq))))) then
         print*,'Disaster4'
         STOP
      endif


      call Do4Minimization
      StrMinuit4=mslep

c      mA=mlepton(1)+mNtlino
c      mB=mlepton(2)+mNtlino
c      mMax=max(mA,mB)
c      
c      if (StrMinuit4.gt.mMax) then
c         StrMinuit4=mMax
c         print *,'4RESETTING ',StrMinuit4
c      else
c         print *,'4NOT'
c      endif
      end
c-----------------------------------------------------------------
      function StrMinuit3(
     &     kkpxlep,
     &     kkpylep,
     &     kketvis,
     &     kkpxvis,
     &     kkpyvis,
     &     kkmlepton,
     &     kkmNtlino)
c-----------------------------------------------------------------
c     This is a single parameter roots minimisation method
c     for calculating MT2 seemingly being an implementation of
c     the TruthBehindTheGlamour
c     equation (25) method.  As such it is only valid for events
c     where the masses of the two principle visible particles are
c     IDENTICAL.  See StrMinuit4 in this file and in str_miniut4.f
c     for extensions to the dissimilar mass case.  Working for the
c     derivation of this method in TBG and in Lab Book 3 near
c     centre (see "Here's the Start"). Lester
c-----------------------------------------------------------------
      implicit none
c-----Defn
      double precision StrMinuit3

c-----Input Variables
      double precision kkpxlep(2)
      double precision kkpylep(2)
      double precision kketvis
      double precision kkpxvis
      double precision kkpyvis
      double precision kkmlepton(2)
      double precision kkmNtlino
c-----Includes
      include 'MINI3.INC'

      call FillMini3BlockWith(
     &     kkpxlep,
     &     kkpylep,
     &     kketvis,
     &     kkpxvis,
     &     kkpyvis,
     &     kkmlepton,
     &     kkmNtlino)
      call Do3Minimization
      StrMinuit3=mslep
      end
c-----------------------------------------------------------------
      function StrMinuit2(kkpxlep,
     &     kkpylep,
     &     kkpzlep,
     &     kkpxmiss,
     &     kkpymiss,
     &     kkmlepton,
     &     kkmNtlino)
c-----------------------------------------------------------------
      implicit none
c-----Defn
      double precision StrMinuit2
c-----Input Variables
      double precision kkpxlep(2)
      double precision kkpylep(2)
      double precision kkpzlep(2)
      double precision kkpxmiss
      double precision kkpymiss
      double precision kkmlepton(2)
      double precision kkmNtlino
c-----Includes
      include 'MINI2.INC'

      call FillMini2BlockWith(kkpxlep,
     &     kkpylep,
     &     kkpzlep,
     &     kkpxmiss,
     &     kkpymiss,
     &     kkmlepton,
     &     kkmNtlino)
      call Do2Minimization
      StrMinuit2=mslep
      end
c-----------------------------------------------------------------
      function StrMinuit(ptl1, ptl2, ptmiss, m1, mlep1, mlep2)
c-----------------------------------------------------------------
      implicit none
c-----Defn
      double precision StrMinuit
c-----Input Variables
      double precision ptl1(2), ptl2(2), ptmiss(2), m1, mlep1, mlep2
c-----Includes
      include 'MINI.INC'

c      print *,ptl1(1),ptl1(2),ptl2(1),ptl2(2),ptmiss(1),ptmiss(2)
c      print *,m1,mlep1,mlep2
c      StrMinuit=ptmiss(2)
c      return
      call FillMiniBlockWith(ptl1,ptl2,ptmiss,m1, mlep1, mlep2)
      call DoMinimization
      StrMinuit=mslep
      end
c-----------------------------------------------------------------
      subroutine Do4Minimization
c-----------------------------------------------------------------
      implicit none
      include 'MINI4.INC'
      double precision arglis
      integer iflag
      external myfcn4
      dimension arglis(10)
c      integer numberResets

      call Reset4Minuit
      
C-----Best config:      
c      arglis(1)=100
c      print *,'MIGRAD ************* 0'
c      call mnexcm(myfcn4,'SEEK',arglis,0,iflag,0)
c      print *,'MIGRAD ************* 1'
c      arglis(1)=0
c      call mnexcm(myfcn4,'MIGRAD',arglis,0,iflag,0)


      arglis(1)=100
*AJB      print *,'MIGRAD ************* 0'
      call mnexcm(myfcn4,'SEEK',arglis,0,iflag,0)
*AJB      print *,'MIGRAD ************* 1'
      arglis(1)=0
      call mnexcm(myfcn4,'MIGRAD',arglis,0,iflag,0)
c      print *,'MIGRAD ************* 2'
c      call mnexcm(myfcn4,'IMPR',arglis,0,iflag,0)
c      if (didReset.and.numberResets.lt.10) goto 10
c      print *,'MIGRAD ************* 3'
c      call mnexcm(myfcn4,'MIGRAD',arglis,0,iflag,0)
c      print *,'MIGRAD ************* 4'
c      call mnexcm(myfcn4,'IMPR',arglis,0,iflag,0)
c      print *,'MIGRAD ************* 6'

C      arglis(1)=0
      call PackResult4
      end
c-----------------------------------------------------------------
      subroutine Do3Minimization
c-----------------------------------------------------------------
      implicit none
      double precision arglis
      integer iflag
      external myfcn3
      dimension arglis(10)

      call Reset3Minuit
      
      arglis(1)=100
      call mnexcm(myfcn3,'SEEK',arglis,0,iflag,0)
      arglis(1)=0
      call mnexcm(myfcn3,'MIGRAD',arglis,0,iflag,0)

      call mnexcm(myfcn3,'IMPR',arglis,0,iflag,0)

C      arglis(1)=0
      call PackResult3
      end
c-----------------------------------------------------------------
      subroutine Do2Minimization
c-----------------------------------------------------------------
      implicit none
      double precision arglis
      integer iflag
      external myfcn2
      dimension arglis(10)

      call Reset2Minuit
      
      arglis(1)=100
      call mnexcm(myfcn2,'SEEK',arglis,0,iflag,0)
      arglis(1)=0
      call mnexcm(myfcn2,'MIGRAD',arglis,0,iflag,0)

      call mnexcm(myfcn2,'IMPR',arglis,0,iflag,0)

C      arglis(1)=0
      call PackResult2
      end
c-----------------------------------------------------------------
      subroutine DoMinimization
c-----------------------------------------------------------------
      implicit none
      double precision arglis
      integer iflag
      external myfcn
      dimension arglis(10)

      call ResetMinuit
      
      arglis(1)=100
      call mnexcm(myfcn,'SEEK',arglis,0,iflag,0)
      arglis(1)=0
      call mnexcm(myfcn,'MIGRAD',arglis,0,iflag,0)

      call mnexcm(myfcn,'IMPR',arglis,0,iflag,0)

C      arglis(1)=0
      call PackResult
      end
c-----------------------------------------------------------------
      subroutine Reset4Minuit
c-----------------------------------------------------------------
      implicit none
c-----Little local variables
      integer i,iflag
c-----Important variables
      integer nprm
      double precision vstrt,stp,lbnd,ubnd
      external myfcn4
      double precision arglis
      dimension arglis(10)

      character*10 pnam(1)
      dimension nprm(1)
      dimension vstrt(1),stp(1),lbnd(1),ubnd(1)

C-----Best config: unbdd vstart 1.0D2 stp 1.0D1
      data nprm /    1        /
      data pnam /   'excess' /
      data vstrt/    1.0D2   /
      data stp  /    1.0D1    /
      data lbnd /    0.0D0    /
      data ubnd /    0.0D0    /
c      data lbnd /    -120.0D0    /
c      data ubnd /    +120.0D0    /

c      print *,'STEP A'
      arglis(1)=-1
      call mninit(5,6,7)
c      print *,'STEP A1'
      call mnexcm(myfcn4,'SET PRI',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set printout-level -1.'
            stop
      endif
      call mnexcm(myfcn4,'SET NOW',arglis,0,iflag,0)
c      print *,'STEP A2'
      if (iflag.ne.0) then
            print *,'Unable to set no-warnings.'
            stop
      endif
c      print *,'STEP A3'
      do i=1,1
c      print *,'STEP A4'
c      print *,nprm(i),pnam(i),vstrt(i),
c     &                       stp(i),lbnd(i),ubnd(i)
         call mnparm(nprm(i),pnam(i),vstrt(i),
     &                       stp(i),lbnd(i),ubnd(i),iflag)
*AJB      print *,'STEP 3A5'
         if (iflag.ne.0) then
            print *,'Unable to set parameter ',i
            stop
*AJB      print *,'STEP 3A6'
         endif
      enddo
c      print *,'STEP A7'

C     set to rigorous strategy
      arglis(1)=2.D0
      call mnexcm(myfcn4,'SET STRATEGY',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set STRATEGY to', arglis(1)
            stop
      endif
      

c      print *,'STEP B'
      call mnseti('This is an easy one 4.')
c      print *,'STEP C'
      end
c-----------------------------------------------------------------
      subroutine Reset3Minuit
c-----------------------------------------------------------------
      implicit none
c-----Little local variables
      integer i,iflag
c-----Important variables
      integer nprm
      double precision vstrt,stp,lbnd,ubnd
      external myfcn3
      double precision arglis
      dimension arglis(10)

      character*10 pnam(1)
      dimension nprm(1)
      dimension vstrt(1),stp(1),lbnd(1),ubnd(1)

      data nprm /    1        /
      data pnam /   'excess' /
      data vstrt/    1.0D6    /
      data stp  /    1.0D5    /
      data lbnd /    0.0D0    /
      data ubnd /    0.0D0    /

c      print *,'STEP A'
      arglis(1)=-1
      call mninit(5,6,7)
c      print *,'STEP A1'
      call mnexcm(myfcn3,'SET PRI',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set printout-level -1.'
            stop
      endif
      call mnexcm(myfcn3,'SET NOW',arglis,0,iflag,0)
c      print *,'STEP A2'
      if (iflag.ne.0) then
            print *,'Unable to set no-warnings.'
            stop
      endif
c      print *,'STEP A3'
      do i=1,1
c      print *,'STEP A4'
c      print *,nprm(i),pnam(i),vstrt(i),
c     &                       stp(i),lbnd(i),ubnd(i)
         call mnparm(nprm(i),pnam(i),vstrt(i),
     &                       stp(i),lbnd(i),ubnd(i),iflag)
*AJB      print *,'STEP 3A5'
         if (iflag.ne.0) then
            print *,'Unable to set parameter ',i
            stop
*AJB      print *,'STEP 3A6'
         endif
      enddo
c      print *,'STEP A7'

C     set to rigorous strategy
      arglis(1)=2.D0
      call mnexcm(myfcn3,'SET STRATEGY',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set STRATEGY to', arglis(1)
            stop
      endif
      

c      print *,'STEP B'
      call mnseti('This is an easy one 3.')
c      print *,'STEP C'
      end
c-----------------------------------------------------------------
      subroutine Reset2Minuit
c-----------------------------------------------------------------
      implicit none
c-----Little local variables
      integer i,iflag
c-----Important variables
      integer nprm
      double precision vstrt,stp,lbnd,ubnd
      external myfcn2
      double precision arglis
      dimension arglis(10)

      character*10 pnam(4)
      dimension nprm(4)
      dimension vstrt(4),stp(4),lbnd(4),ubnd(4)

      data nprm /    1    ,   2    ,   3    ,   4     /
      data pnam /   'pxn1',  'pyn1',  'pzn1',  'pzn2' /
      data vstrt/    0.0D0,   0.0D0,   0.0D0,   0.0D0 /
      data stp  /    1.0D1,   1.0D1,   1.0D1,   1.0D1 /
      data lbnd /    0.0D0,   0.0D0,   0.0D0,   0.0D0 /
      data ubnd /    0.0D0,   0.0D0,   0.0D0,   0.0D0 /

c      print *,'STEP A'
      arglis(1)=-1
      call mninit(5,6,7)
c      print *,'STEP A1'
      call mnexcm(myfcn2,'SET PRI',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set printout-level -1.'
            stop
      endif
      call mnexcm(myfcn2,'SET NOW',arglis,0,iflag,0)
c      print *,'STEP A2'
      if (iflag.ne.0) then
            print *,'Unable to set no-warnings.'
            stop
      endif
c      print *,'STEP A3'
      do i=1,4
c      print *,'STEP A4'
c      print *,nprm(i),pnam(i),vstrt(i),
c     &                       stp(i),lbnd(i),ubnd(i)
         call mnparm(nprm(i),pnam(i),vstrt(i),
     &                       stp(i),lbnd(i),ubnd(i),iflag)
*AJB      print *,'STEP 2A5'
         if (iflag.ne.0) then
            print *,'Unable to set parameter ',i
            stop
*AJB      print *,'STEP 2A6'
         endif
      enddo
c      print *,'STEP A7'

C     set to rigorous strategy
      arglis(1)=2.D0
      call mnexcm(myfcn2,'SET STRATEGY',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set STRATEGY to', arglis(1)
            stop
      endif
      

c      print *,'STEP B'
      call mnseti('This is an easy one 2.')
c      print *,'STEP C'
      end
c-----------------------------------------------------------------
      subroutine ResetMinuit
c-----------------------------------------------------------------
      implicit none
c-----Little local variables
      integer i,iflag
c-----Important variables
      integer nprm
      double precision vstrt,stp,lbnd,ubnd
      external myfcn
      double precision arglis
      dimension arglis(10)

      character*10 pnam(2)

      dimension nprm(2)
      dimension vstrt(2),stp(2),lbnd(2),ubnd(2)

      data nprm /    1     ,   2     /
      data pnam /   'pxn1' ,  'pyn1' /
      data vstrt/    0.0D0 ,   0.0D0 /
      data stp  /    1.0D1 ,   1.0D1 /
      data lbnd /    0.0D0 ,   0.0D0 /
      data ubnd /    0.0D0 ,   0.0D0 /

c      print *,'STEP A'
      arglis(1)=-1
      call mninit(5,6,7)
c      print *,'STEP A1'
      call mnexcm(myfcn,'SET PRI',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set printout-level -1.'
            stop
      endif
      call mnexcm(myfcn,'SET NOW',arglis,0,iflag,0)
c      print *,'STEP A2'
      if (iflag.ne.0) then
            print *,'Unable to set no-warnings.'
            stop
      endif
c      print *,'STEP A3'
      do i=1,2
c      print *,'STEP A4'
c      print *,nprm(i),pnam(i),vstrt(i),
c     &                       stp(i),lbnd(i),ubnd(i)
         call mnparm(nprm(i),pnam(i),vstrt(i),
     &                       stp(i),lbnd(i),ubnd(i),iflag)
*AJB      print *,'STEP 1A5'
         if (iflag.ne.0) then
            print *,'Unable to set parameter ',i
            stop
*AJB      print *,'STEP 1A6'
         endif
      enddo
c      print *,'STEP A7'

C     set to rigorous strategy
      arglis(1)=2.D0
      call mnexcm(myfcn,'SET STRATEGY',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set STRATEGY to', arglis(1)
            stop
      endif
      

c      print *,'STEP B'
      call mnseti('This is an easy one.')
c      print *,'STEP C'
      end
c-----------------------------------------------------------------
      subroutine FillMini4BlockWith(kkpxlep,
     &                              kkpylep,
     &                              kketvis,
     &                              kkpxvis,
     &                              kkpyvis,
     &                              kkmlepton,
     &                              kkmNtlino)
c-----------------------------------------------------------------
      implicit none
c-----Common blocks
      include 'MINI4.INC'
c-----Input Variables
      double precision kkpxlep(2)
      double precision kkpylep(2)
      double precision kketvis
      double precision kkpxvis
      double precision kkpyvis
      double precision kkmlepton(2)
      double precision kkmNtlino
c-----local vars
c      integer i
      double precision etnab
      double precision pxnab
      double precision pynab
      double precision etsig
      double precision pxsig
      double precision pysig
      
c      validGood=.false.

      pxlep(1)=kkpxlep(1)
      pylep(1)=kkpylep(1)

      pxlep(2)=kkpxlep(2)
      pylep(2)=kkpylep(2)

      mlepton(1)=kkmlepton(1)
      mlepton(2)=kkmlepton(2)
      
      etlep(1)=sqrt(pxlep(1)**2+pylep(1)**2+mlepton(1)**2)
      etlep(2)=sqrt(pxlep(2)**2+pylep(2)**2+mlepton(2)**2)

      etvis=kketvis
      pxvis=kkpxvis
      pyvis=kkpyvis

C      write(*,*) "Total visible (E,x,y)",etvis,pxvis,pyvis

      etsig=etlep(1)+etlep(2)
      pxsig=pxlep(1)+pxlep(2)
      pysig=pylep(1)+pylep(2)

      etnab=etlep(1)-etlep(2)
      pxnab=pxlep(1)-pxlep(2)
      pynab=pylep(1)-pylep(2)

      mNtlino=kkmNtlino

      lambdaDotBigSigma   =  etvis
      lambdaDotSmallSigma =  etsig
      lambdaDotNabla      =  etnab
      nablaDotBigSigma    =  etnab*etvis
     &                     - pxnab*pxvis
     &                     - pynab*pyvis
      nablaDotSmallSigma  =  etnab*etsig
     &                     - pxnab*pxsig
     &                     - pynab*pysig
      bigSigmaSquared     =  etvis**2 
     &                     - pxvis**2
     &                     - pyvis**2
      smallSigmaDotBigSigma =  etsig*etvis
     &                       - pxsig*pxvis
     &                       - pysig*pyvis
      smallSigmaSquared   =  etsig**2 
     &                     - pxsig**2
     &                     - pysig**2
      nablaSquared        =  etnab**2 
     &                     - pxnab**2
     &                     - pynab**2
      smallSigmaDotNabla = etsig*etnab
     &                     - pxsig*pxnab
     &                     - pysig*pynab
      
      alphaSqPlusBetaSq = mlepton(1)**2+mlepton(2)**2

      rootSMin=lambdaDotBigSigma+sqrt(
     &     4.0*mNtlino**2+lambdaDotBigSigma**2-bigSigmaSquared
     &                               )

      if(.true.) then
         write (*,1023)'lambdaDotBigSigma     ', lambdaDotBigSigma
         write (*,1023)'lambdaDotSmallSigma   ', lambdaDotSmallSigma
         write (*,1023)'lambdaDotNabla        ', lambdaDotNabla
         write (*,1023)'nablaDotBigSigma      ', nablaDotBigSigma
         write (*,1023)'nablaDotSmallSigma    ', nablaDotSmallSigma
         write (*,1023)'smallSigmaDotBigSigma ', smallSigmaDotBigSigma
         write (*,1023)'smallSigmaSquared     ', smallSigmaSquared
         write (*,1023)'bigSigmaSquared       ', bigSigmaSquared
         write (*,1023)'nablaSquared          ', nablaSquared
         write (*,1023)'rootSMin              ', rootSMin
         write (*,1023)'smallSigmaDotNabla    ', smallSigmaDotNabla
         write (*,1023)'alphaSqPlusBetaSq     ', alphaSqPlusBetaSq
      endif
      
 1023 FORMAT(A,F14.5)
      end
c-----------------------------------------------------------------
      subroutine FillMini3BlockWith(kkpxlep,
     &                              kkpylep,
     &                              kketvis,
     &                              kkpxvis,
     &                              kkpyvis,
     &                              kkmlepton,
     &                              kkmNtlino)
c-----------------------------------------------------------------
      implicit none
c-----Common blocks
      include 'MINI3.INC'
c-----Input Variables
      double precision kkpxlep(2)
      double precision kkpylep(2)
      double precision kketvis
      double precision kkpxvis
      double precision kkpyvis
      double precision kkmlepton(2)
      double precision kkmNtlino
c-----local vars
c      integer i
      double precision etnab
      double precision pxnab
      double precision pynab
      double precision etsig
      double precision pxsig
      double precision pysig
      
      pxlep(1)=kkpxlep(1)
      pylep(1)=kkpylep(1)

      pxlep(2)=kkpxlep(2)
      pylep(2)=kkpylep(2)

      mlepton(1)=kkmlepton(1)
      mlepton(2)=kkmlepton(2)
      
      etlep(1)=sqrt(pxlep(1)**2+pylep(1)**2+mlepton(1)**2)
      etlep(2)=sqrt(pxlep(2)**2+pylep(2)**2+mlepton(2)**2)

      etvis=kketvis
      pxvis=kkpxvis
      pyvis=kkpyvis

      etsig=etlep(1)+etlep(2)
      pxsig=pxlep(1)+pxlep(2)
      pysig=pylep(1)+pylep(2)

      etnab=etlep(1)-etlep(2)
      pxnab=pxlep(1)-pxlep(2)
      pynab=pylep(1)-pylep(2)

      mNtlino=kkmNtlino

      lambdaDotBigSigma   =  etvis
      lambdaDotSmallSigma =  etsig
      lambdaDotNabla      =  etnab
      nablaDotBigSigma    =  etnab*etvis
     &                     - pxnab*pxvis
     &                     - pynab*pyvis
      nablaDotSmallSigma  =  etnab*etsig
     &                     - pxnab*pxsig
     &                     - pynab*pysig
      bigSigmaSquared     =  etvis**2 
     &                     - pxvis**2
     &                     - pyvis**2
      smallSigmaDotBigSigma =  etsig*etvis
     &                       - pxsig*pxvis
     &                       - pysig*pyvis
      smallSigmaSquared   =  etsig**2 
     &                     - pxsig**2
     &                     - pysig**2
      nablaSquared        =  etnab**2 
     &                     - pxnab**2
     &                     - pynab**2
      rootSMin=lambdaDotBigSigma+sqrt(
     &     4.0*mNtlino**2+lambdaDotBigSigma**2-bigSigmaSquared
     &                               )
      end
c-----------------------------------------------------------------
      subroutine FillMini2BlockWith(kkpxlep,
     &                              kkpylep,
     &                              kkpzlep,
     &                              kkpxmiss,
     &                              kkpymiss,
     &                              kkmlepton,
     &                              kkmNtlino)
c-----------------------------------------------------------------
      implicit none
c-----Common blocks
      include 'MINI2.INC'
c-----Input Variables
      double precision kkpxlep(2)
      double precision kkpylep(2)
      double precision kkpzlep(2)
      double precision kkpxmiss
      double precision kkpymiss
      double precision kkmlepton(2)
      double precision kkmNtlino
c-----local vars
c      integer i
      
      pxlep(1)=kkpxlep(1)
      pylep(1)=kkpylep(1)
      pzlep(1)=kkpzlep(1)

      pxlep(2)=kkpxlep(2)
      pylep(2)=kkpylep(2)
      pzlep(2)=kkpzlep(2)

      pmissx=kkpxmiss
      pmissy=kkpymiss

      mlepton(1)=kkmlepton(1)
      mlepton(2)=kkmlepton(2)

      mNtlino=kkmNtlino

      end
c-----------------------------------------------------------------
      subroutine FillMiniBlockWith(ptl1,ptl2,ptmiss,m1, mlep1, mlep2)
c-----------------------------------------------------------------
      implicit none
c-----Common blocks
      include 'MINI.INC'
c-----Input Variables
      double precision ptl1(2), ptl2(2), ptmiss(2), m1, mlep1, mlep2

      pmissx=ptmiss(1)
      pmissy=ptmiss(2)
      pxlep(1)=ptl1(1)
      pylep(1)=ptl1(2)
      pxlep(2)=ptl2(1)
      pylep(2)=ptl2(2)
      mntlino=m1
      mlepton(1)=mlep1
      mlepton(2)=mlep2
      end
c-----------------------------------------------------------------
      subroutine FillMiniBlockWithConstants
c-----------------------------------------------------------------
      implicit none
      include 'MINI.INC'

      pmissx=-20
      pmissy=30
      pxlep(1)=-40
      pylep(1)=0
      pxlep(2)=60
      pylep(2)=30
      mntlino=40
      mlepton(1)=5
      mlepton(2)=5
      end
c-----------------------------------------------------------------
      subroutine PackResult4
c-----------------------------------------------------------------
      implicit none
      include 'MINI4.INC'
      
      integer iger1,iger2,iger3
      double precision doub0,doub1,doub2
      
      call mnstat(doub0,doub1,doub2,iger1,iger2,iger3)
      mslep=doub0
      end
c-----------------------------------------------------------------
      subroutine PackResult3
c-----------------------------------------------------------------
      implicit none
      include 'MINI3.INC'
      
      integer iger1,iger2,iger3
      double precision doub0,doub1,doub2
      
      call mnstat(doub0,doub1,doub2,iger1,iger2,iger3)
      mslep=doub0
      end
c-----------------------------------------------------------------
      subroutine PackResult2
c-----------------------------------------------------------------
      implicit none
      include 'MINI2.INC'
      
      integer iger1,iger2,iger3
      double precision doub0,doub1,doub2
      
      call mnstat(doub0,doub1,doub2,iger1,iger2,iger3)
      mslep=doub0
      end
c-----------------------------------------------------------------
      subroutine PackResult
c-----------------------------------------------------------------
      implicit none
      include 'MINI.INC'
      
      integer iger1,iger2,iger3
      double precision doub0,doub1,doub2
      
      call mnstat(doub0,doub1,doub2,iger1,iger2,iger3)
      mslep=doub0
      end
c-----------------------------------------------------------------
      subroutine myfcn4(npar,grad,f,x,iflag,futil)
c-----------------------------------------------------------------
      implicit none
c-----Foreign params
      include 'MINI4.INC'
c-----Input params
      integer npar,iflag
      double precision grad,f,x
c-----Output params
      double precision futil
c-----Dimensions
      dimension grad(*),x(*)
c-----Local vars
      double precision sigDotB,BSQ,rootS,
     &                inner1,inner2,ttp,H
      double precision tt1,tt2,nablaDotB,eAlpha,eBeta,
     &                                   pAlpha,pBeta,
     &                    eChi,pChi

    
      rootS=rootSMin+x(1)**2.0D0

      BSQ = roots**2.0D0
     &     +bigSigmaSquared-2.0D0*rootS*lambdaDotBigSigma

      sigDotB = lambdaDotSmallSigma*rootS-smallSigmaDotBigSigma


      H = sigDotB**2.0D0-smallSigmaSquared*BSQ  
      nablaDotB=lambdaDotNabla*rootS-nablaDotBigSigma

c      Q=1.0D0-(nablaDotB**2)/H

      tt1 = nablaDotB**2.0D0*smallSigmaSquared
     &     + smallSigmaDotNabla**2.0D0*BSQ
     &     - 2.0D0*nablaDotB*smallSigmaDotNabla*sigDotB

      tt2 = (nablaDotB+smallSigmaDotNabla)**2.0D0

c      Q1=(1.0D0-tt1/(H*smallSigmaSquared))
c      Q2=(1.0D0-tt2/H)

      inner1=(BSQ-4.0D0*mNtlino**2.0D0)*H-BSQ*tt2
      inner2=H*(-nablaSquared)-tt1
c  alphaSqPlusBetaSq
      

      if (inner1.lt.0) then
c--------This means that cosThetaSq>1 so no balance.pt. soln, so

         eAlpha=0.5D0*(sigDotB+nablaDotB)/sqrt(BSQ)
         eBeta =0.5D0*(sigDotB-nablaDotB)/sqrt(BSQ)
         
         pAlpha=sqrt(abs(eAlpha**2-mlepton(1)**2))
         pBeta =sqrt(abs(eBeta **2-mlepton(2)**2))

         eChi=0.5D0*sqrt(BSQ)
         pChi=sqrt(abs(eChi**2-mNtlino**2))

         f=Max((eChi+eAlpha)**2-(pChi+pAlpha)**2,
     &         (eChi*eBeta )**2-(pChi+pBeta )**2 )

         f=sqrt(abs(f))
*AJB         print *,'SAW4 SPECIAL: f=',f

         return
c         print *,'THING4: NUMERICAL ACCURACY WARNING: inner1=',
c     &               inner1,'<0, BSQ-4X^2 = ',
c     &              BSQ-4.0D0*mNtlino**2.0D0,' !'
c         print *,'nabDotB ',nablaDotB,'   smallSigmaDotNabla ',
c     &             smallSigmaDotNabla
c         print *,'cthSq = ',
c     &           BSQ*tt2/((BSQ-4.0D0*mNtlino**2.0D0)*(H))
c     &     , '   H=',H
         
      endif
      if (inner2.lt.0) then
         print *,'THING4: NUMERICAL ACCURACY WARNING: inner2=',
     &               inner2,'<0 !'
         inner2=0
      endif


      f =  mNtlino**2+0.5*(alphaSqPlusBetaSq)
     &     +0.5*sigDotB-(0.5/H)*(
     &     (nablaDotB+smallSigmaDotNabla)*
     &       (nablaDotB*sigDotB-BSQ*smallSigmaDotNabla)
     &       +
     &       sqrt(inner1*inner2)
     &     )
      
 
c      if (f.ge.0) then
c         if (validGood) then
c            goodRootS=min(rootS,goodRootS)
c         else
c            goodRootS=rootS
c            validGood=.true.
c         endif
c         print *,"good=",goodRootS
c      endif
      

      if (f.lt.0) then
c         if (validGood.and.rootS.lt.goodRootS) then
c            rootSMin=rootS
c         endif
c         didReset=.true.
c         ttp=0
c         if (f<0) then
         print *,'PREBAD4 f ',f
         ttp=ttp-f;
         f=abs(f)*exp(ttp)
         print *,'BAD4 f ',f
c         endif
c         if (inner1<0) then
c            ttp=ttp-inner1;
c         endif
c         if (inner2<0) then
c            ttp=ttp-inner2;
c         endif
      endif

      f=sqrt(f)
*AJB      print*,'COMMENT4 ',f,inner1,inner2,H,nablaDotB

      end
c-----------------------------------------------------------------
      subroutine myfcn3(npar,grad,f,x,iflag,futil)
c-----------------------------------------------------------------
      implicit none
c-----Foreign params
      include 'MINI3.INC'
c-----Input params
      integer npar,iflag
      double precision grad,f,x
c-----Output params
      double precision futil
c-----Dimensions
      dimension grad(*),x(*)
c-----Local vars
      double precision Q,sigDotB,BSQ,rootS,inner1,inner2,ttp

      rootS=rootSMin+x(1)**2

      BSQ = roots**2+bigSigmaSquared-2.0*rootS*lambdaDotBigSigma

      sigDotB = lambdaDotSmallSigma*rootS-smallSigmaDotBigSigma

      Q=1.0D0-(
     &      (lambdaDotNabla*rootS-nablaDotBigSigma)**2
     &       /
     &       ( sigDotB**2-smallSigmaSquared*BSQ  ))


      inner1=(BSQ*Q-4.0*mNtlino**2)
      inner2=(smallSigmaSquared*Q-4.0*mlepton(1)**2)
      

      if (nablaSquared.gt.0) then
         print *,'NUMERICAL ACCURACY WARNING: NABLASQ=',
     &               nablaSquared,'>0!'
      endif
      if (inner1.lt.0) then
         print *,'NUMERICAL ACCURACY WARNING: inner1=',
     &               inner1,'<0!'
      endif
      if (inner2.lt.0) then
         print *,'NUMERICAL ACCURACY WARNING: inner2=',
     &               inner2,'<0!'
      endif


      if (inner1.ge.0.and.inner2.ge.0) then
         f =   mlepton(1)**2
     &        + mNtlino**2
     &        + 0.5*(
     &        sigDotB*Q - sqrt(
     &        inner1*inner2
     &        )
     &        )
      print *,'OK'
      else
         ttp=0
         if (inner1.lt.0) then
            ttp=ttp-inner1
         endif
         if (inner2.lt.0) then
            ttp=ttp-inner2
         endif
         f =   (mlepton(1)**2
     &        + mNtlino**2
     &        + 0.5*(
     &        sigDotB*Q + sqrt(
     &        abs(inner1*inner2)
     &        )
     &        ))*exp(ttp)
      endif

      f=sqrt(f)

      end
c-----------------------------------------------------------------
      subroutine myfcn2(npar,grad,f,x,iflag,futil)
c-----------------------------------------------------------------
      implicit none
c-----Foreign params
      include 'MINI2.INC'
c-----Local vars
      double precision pxneu,pyneu,pzneu
      double precision msq
      integer i
c-----Input params
      integer npar,iflag
      double precision grad,f,x
c-----Output params
      double precision futil
c-----Dimensions
      dimension pxneu(2),pyneu(2),pzneu(2),msq(2)
      dimension grad(*),x(*)

c      f=(x(1)-3.0D0)*(x(1)-3.0D0)+(x(2)-2.0D0)*(x(2)-2.0D0)+7.0D0
c      f=sin(x(1))+0.1D0*(x(2)**2.0D0)

c-----x(1) and x(2) can determine px and py of one neutralino.
c-----Given this splitting, derive px and py of other ntlino too.
      pxneu(1)=x(1)
      pyneu(1)=x(2)
      pzneu(1)=x(3)
      
      pxneu(2)=pmissx-pxneu(1)
      pyneu(2)=pmissy-pyneu(1)
      pzneu(2)=x(4)
      
      do i=1,2
         msq(i)=(mntlino**2) + mlepton(i)**2 + 2.0D0*(
     &      sqrt(pxneu(i)**2+
     &           pyneu(i)**2+
     &           pzneu(i)**2+
     &           mntlino**2)
     &        *
     &      sqrt(pxlep(i)**2+
     &           pylep(i)**2+
     &           pzlep(i)**2+
     &           mlepton(i)**2)
     &        -
     &      (pxneu(i)*pxlep(i)+
     &       pyneu(i)*pylep(i)+
     &       pzneu(i)*pzlep(i))
     &    )
      enddo

      if (msq(1).ge.msq(2)) then
         f=sqrt(msq(1))
      else
         f=sqrt(msq(2))
      endif

      return
      end
c-----------------------------------------------------------------
      subroutine myfcn(npar,grad,f,x,iflag,futil)
c-----------------------------------------------------------------
      implicit none
c-----Foreign params
      include 'MINI.INC'
c-----Local vars
      double precision pxneu,pyneu
      double precision msq
      integer i
c-----Input params
      integer npar,iflag
      double precision grad,f,x
c-----Output params
      double precision futil
c-----Dimensions
      dimension pxneu(2),pyneu(2),msq(2)
      dimension grad(*),x(*)

c      f=(x(1)-3.0D0)*(x(1)-3.0D0)+(x(2)-2.0D0)*(x(2)-2.0D0)+7.0D0
c      f=sin(x(1))+0.1D0*(x(2)**2.0D0)

c-----x(1) and x(2) can determine px and py of one neutralino.
c-----Given this splitting, derive px and py of other ntlino too.
      pxneu(1)=x(1)
      pyneu(1)=x(2)
      
      pxneu(2)=pmissx-pxneu(1)
      pyneu(2)=pmissy-pyneu(1)
      
      do i=1,2
         msq(i)=(mntlino**2) + mlepton(i)**2 + 2.0D0*(
     &      sqrt(pxneu(i)**2+pyneu(i)**2+mntlino**2)
     &        *
     &      sqrt(pxlep(i)**2+pylep(i)**2+mlepton(i)**2)
     &        -
     &      (pxneu(i)*pxlep(i)+pyneu(i)*pylep(i))
     &    )
      enddo

      if (msq(1).ge.msq(2)) then
         f=sqrt(msq(1))
      else
         f=sqrt(msq(2))
      endif

      return
      end
c-----------------------------------------------------------------
