c-----------------------------------------------------------------
      function StrMinuit4(
     &     kkpxlep,
     &     kkpylep,
     &     kketvis,
     &     kkpxvis,
     &     kkpyvis,
     &     kkmlepton,
     &     kkmNtlino)
c-----------------------------------------------------------------
c     This is the single parameter roots minimisation method
c     for calculating MT2 analogous to the TruthBehindTheGlamour
c     equation (25) method, but extended to be able to deal
c     with visible particles of DISSIMILAR masses.  Never written
c     up in a published article, but initially documented in
c     Lester Home Lab Book (near centre) and then re-documented in
c     Lester Lab Book 6 on 4th and 5th July 2007.
c     This method is implemented identically (but with more 
c     comments and more debug output) in the separate file
c     str_minuit.f ... and is also copied to str_minuit5.f with
c     a name change for no apparent reason.
c-----------------------------------------------------------------
      implicit none
c-----Defn
      double precision StrMinuit4
c-----Input Variables
      double precision kkpxlep(2)
      double precision kkpylep(2)
      double precision kketvis
      double precision kkpxvis
      double precision kkpyvis
      double precision kkmlepton(2)
      double precision kkmNtlino
      double precision n1aSq,n1bSq,n2aSq,n2bSq,
     &                 p2ax,p2ay,p2bx,p2by,p2aSq,p2bSq
c-----Includes
      include 'MINI4.INC'

      call FillMini4BlockWith(
     &     kkpxlep,
     &     kkpylep,
     &     kketvis,
     &     kkpxvis,
     &     kkpyvis,
     &     kkmlepton,
     &     kkmNtlino)

      p2ax=-pxvis-mNtlino*pxlep(1)/mlepton(1)
      p2ay=-pyvis-mNtlino*pylep(1)/mlepton(1)
      p2bx=-pxvis-mNtlino*pxlep(2)/mlepton(2)
      p2by=-pyvis-mNtlino*pylep(2)/mlepton(2)

      p2aSq=p2ax**2+p2ay**2
      p2bSq=p2bx**2+p2by**2

      n1aSq=(mlepton(1)+mNtlino)**2
      n1bSq=(mlepton(2)+mNtlino)**2

      n2aSq=(mlepton(2))**2+mNtlino**2+2.0D0*(
     &             etlep(2)*sqrt(mNtlino**2+p2aSq)
     &            -pxlep(2)*p2ax
     &            -pylep(2)*p2ay
     &           )
      n2bSq=(mlepton(1))**2+mNtlino**2+2.0D0*(
     &             etlep(1)*sqrt(mNtlino**2+p2bSq)
     &            -pxlep(1)*p2bx
     &            -pylep(1)*p2by
     &           )

C-----First account for non-crossing point solutions!

      if (n1aSq.ge.n2aSq) then
c---------Real answer occurs when a's ntlino is at rest wrt a.
         StrMinuit4=sqrt(abs(n1aSq))
         return
      endif
      if (n1bSq.ge.n2bSq) then
c---------Real answer occurs when b's ntlino is at rest wrt b.
         StrMinuit4=sqrt(abs(n1bsq))
         return
      endif
      if (n1aSq.ge.n2aSq.and.n1bSq.ge.n2bSq
     &          .and.(.not.(sqrt(abs(n1aSq)).eq.sqrt(abs(n1bsq))))) then
         print*,'Disaster4'
         STOP
      endif


      call Do4Minimization
      StrMinuit4=mslep
      end
c-----------------------------------------------------------------
      subroutine Do4Minimization
c-----------------------------------------------------------------
      implicit none
      include 'MINI4.INC'
      double precision arglis
      integer iflag
      external myfcn4
      dimension arglis(10)
c      integer numberResets

      call Reset4Minuit
      
      arglis(1)=100
      call mnexcm(myfcn4,'SEEK',arglis,0,iflag,0)
      arglis(1)=0
      call mnexcm(myfcn4,'MIGRAD',arglis,0,iflag,0)

      call PackResult4
      end
c-----------------------------------------------------------------
      subroutine Reset4Minuit
c-----------------------------------------------------------------
      implicit none
c-----Little local variables
      integer i,iflag
c-----Important variables
      integer nprm
      double precision vstrt,stp,lbnd,ubnd
      external myfcn4
      double precision arglis
      dimension arglis(10)

      character*10 pnam(1)
      dimension nprm(1)
      dimension vstrt(1),stp(1),lbnd(1),ubnd(1)

C-----Best config: unbdd vstart 1.0D2 stp 1.0D1
      data nprm /    1        /
      data pnam /   'excess' /
      data vstrt/    1.0D2   /
      data stp  /    1.0D1    /
      data lbnd /    0.0D0    /
      data ubnd /    0.0D0    /

      arglis(1)=-1
      call mninit(5,6,7)
      call mnexcm(myfcn4,'SET PRI',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set printout-level -1.'
            stop
      endif
      call mnexcm(myfcn4,'SET NOW',arglis,0,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set no-warnings.'
            stop
      endif
      do i=1,1
         call mnparm(nprm(i),pnam(i),vstrt(i),
     &                       stp(i),lbnd(i),ubnd(i),iflag)
         if (iflag.ne.0) then
            print *,'Unable to set parameter ',i
            stop
         endif
      enddo

C     set to rigorous strategy
      arglis(1)=2.D0
      call mnexcm(myfcn4,'SET STRATEGY',arglis,1,iflag,0)
      if (iflag.ne.0) then
            print *,'Unable to set STRATEGY to', arglis(1)
            stop
      endif
      

      call mnseti('This is an easy one 4.')
      end
c-----------------------------------------------------------------
      subroutine FillMini4BlockWith(kkpxlep,
     &                              kkpylep,
     &                              kketvis,
     &                              kkpxvis,
     &                              kkpyvis,
     &                              kkmlepton,
     &                              kkmNtlino)
c-----------------------------------------------------------------
      implicit none
c-----Common blocks
      include 'MINI4.INC'
c-----Input Variables
      double precision kkpxlep(2)
      double precision kkpylep(2)
      double precision kketvis
      double precision kkpxvis
      double precision kkpyvis
      double precision kkmlepton(2)
      double precision kkmNtlino
c-----local vars
c      integer i
      double precision etnab
      double precision pxnab
      double precision pynab
      double precision etsig
      double precision pxsig
      double precision pysig
      
c      validGood=.false.

      pxlep(1)=kkpxlep(1)
      pylep(1)=kkpylep(1)

      pxlep(2)=kkpxlep(2)
      pylep(2)=kkpylep(2)

      mlepton(1)=kkmlepton(1)
      mlepton(2)=kkmlepton(2)
      
      etlep(1)=sqrt(pxlep(1)**2+pylep(1)**2+mlepton(1)**2)
      etlep(2)=sqrt(pxlep(2)**2+pylep(2)**2+mlepton(2)**2)

      etvis=kketvis
      pxvis=kkpxvis
      pyvis=kkpyvis

      etsig=etlep(1)+etlep(2)
      pxsig=pxlep(1)+pxlep(2)
      pysig=pylep(1)+pylep(2)

      etnab=etlep(1)-etlep(2)
      pxnab=pxlep(1)-pxlep(2)
      pynab=pylep(1)-pylep(2)

      mNtlino=kkmNtlino

      lambdaDotBigSigma   =  etvis
      lambdaDotSmallSigma =  etsig
      lambdaDotNabla      =  etnab
      nablaDotBigSigma    =  etnab*etvis
     &                     - pxnab*pxvis
     &                     - pynab*pyvis
      nablaDotSmallSigma  =  etnab*etsig
     &                     - pxnab*pxsig
     &                     - pynab*pysig
      bigSigmaSquared     =  etvis**2 
     &                     - pxvis**2
     &                     - pyvis**2
      smallSigmaDotBigSigma =  etsig*etvis
     &                       - pxsig*pxvis
     &                       - pysig*pyvis
      smallSigmaSquared   =  etsig**2 
     &                     - pxsig**2
     &                     - pysig**2
      nablaSquared        =  etnab**2 
     &                     - pxnab**2
     &                     - pynab**2
      smallSigmaDotNabla = etsig*etnab
     &                     - pxsig*pxnab
     &                     - pysig*pynab
      
      alphaSqPlusBetaSq = mlepton(1)**2+mlepton(2)**2

      rootSMin=lambdaDotBigSigma+sqrt(
     &     4.0*mNtlino**2+lambdaDotBigSigma**2-bigSigmaSquared
     &                               )
      end
c-----------------------------------------------------------------
      subroutine PackResult4
c-----------------------------------------------------------------
      implicit none
      include 'MINI4.INC'
      
      integer iger1,iger2,iger3
      double precision doub0,doub1,doub2
      
      call mnstat(doub0,doub1,doub2,iger1,iger2,iger3)
      mslep=doub0
      end
c-----------------------------------------------------------------
      subroutine myfcn4(npar,grad,f,x,iflag,futil)
c-----------------------------------------------------------------
      implicit none
c-----Foreign params
      include 'MINI4.INC'
c-----Input params
      integer npar,iflag
      double precision grad,f,x
c-----Output params
      double precision futil
c-----Dimensions
      dimension grad(*),x(*)
c-----Local vars
      double precision sigDotB,BSQ,rootS,
     &                inner1,inner2,ttp,H
      double precision tt1,tt2,nablaDotB,eAlpha,eBeta,
     &                                   pAlpha,pBeta,
     &                    eChi,pChi

    
      rootS=rootSMin+x(1)**2.0D0

      BSQ = roots**2.0D0
     &     +bigSigmaSquared-2.0D0*rootS*lambdaDotBigSigma

      sigDotB = lambdaDotSmallSigma*rootS-smallSigmaDotBigSigma


      H = sigDotB**2.0D0-smallSigmaSquared*BSQ  
      nablaDotB=lambdaDotNabla*rootS-nablaDotBigSigma

      tt1 = nablaDotB**2.0D0*smallSigmaSquared
     &     + smallSigmaDotNabla**2.0D0*BSQ
     &     - 2.0D0*nablaDotB*smallSigmaDotNabla*sigDotB

      tt2 = (nablaDotB+smallSigmaDotNabla)**2.0D0

      inner1=(BSQ-4.0D0*mNtlino**2.0D0)*H-BSQ*tt2
      inner2=H*(-nablaSquared)-tt1

      if (inner1.lt.0) then
c--------This means that cosThetaSq>1 so no balance.pt. soln, so

         eAlpha=0.5D0*(sigDotB+nablaDotB)/sqrt(BSQ)
         eBeta =0.5D0*(sigDotB-nablaDotB)/sqrt(BSQ)
         
         pAlpha=sqrt(abs(eAlpha**2-mlepton(1)**2))
         pBeta =sqrt(abs(eBeta **2-mlepton(2)**2))

         eChi=0.5D0*sqrt(BSQ)
         pChi=sqrt(abs(eChi**2-mNtlino**2))

         f=Max((eChi+eAlpha)**2-(pChi+pAlpha)**2,
     &         (eChi*eBeta )**2-(pChi+pBeta )**2 )

         f=sqrt(abs(f))
         return
      endif
      if (inner2.lt.0) then
         inner2=0
      endif


      f =  mNtlino**2+0.5*(alphaSqPlusBetaSq)
     &     +0.5*sigDotB-(0.5/H)*(
     &     (nablaDotB+smallSigmaDotNabla)*
     &       (nablaDotB*sigDotB-BSQ*smallSigmaDotNabla)
     &       +
     &       sqrt(inner1*inner2)
     &     )
      
 
      if (f.lt.0) then
         ttp=ttp-f;
         f=abs(f)*exp(ttp)
      endif

      f=sqrt(f)
      end
c-----------------------------------------------------------------
