#include <iostream>
#include "Mt2/Mt2Calculator.h"
//#include "Mt2/Mt2MinimiserNag.h"
//#include "Mt2/Mt2MinimiserGsl.h"
#include "Mt2/Basic_Mt2_332_Calculator.h"
#include "Mt2/SUSYPhys_Mt2_222_Calculator.h"

const double MeV=0.001;
const double GeV=1000*MeV;
const double TeV=1000*GeV;
// so unformatted output will be in GeV

int main(){
  using namespace Mt2;
  using namespace std;

  LorentzTransverseVector A(1., 1.);
  cout << "A = " << A << endl; 
  cout << "A.mass() = " << A.mass() << endl;  
  cout << "A.pt() = " << A.pt() << endl;  
  cout << "A.Et() = " << A.Et() << endl;  
  TwoVector A_tr = A.transverse();
  cout << "A.transverse() = " << A_tr << endl;  
  

  LorentzTransverseVector B(-1., 0.);
  cout << "B = " << B << endl;

  cout << "A+B = " << (A+B) << endl;
  cout << "(A+B).pt() = " << (A+B).pt() << endl;
  cout << "(A+B).Et() = " << (A+B).Et() << endl;
  cout << "(A+B).mass() = " << (A+B).mass() << endl;
  cout << "(A+B).transverse() = " << (A+B).transverse() << endl;  
  cout << "A.transverse() + B.transverse() = " <<  (A.transverse()+B.transverse()) << endl;
  
  cout << "(A+B)*2 = " << (A+B)*2 << endl;

//--------
  LorentzTransverseVector C(TwoVector(-0.1, 0.1), 1.);
  cout << "C = " << C << endl;

  cout << "A+B-A (=B) = " << (A + B - A) << endl;

  LorentzTransverseVector AplusBminusC = (A+B)-C;
  cout << "(A+B)-C = " << AplusBminusC << endl;
  LorentzTransverseVector AplusBminusC2 = A+(B-C);
  cout << "A+(B-C) = " << AplusBminusC2 << endl;
  cout << "(A+B-C).pt() = " << AplusBminusC.pt() << endl;
  cout << "(A+B-C).Et() = " << AplusBminusC.Et() << endl;
  cout << "(A+B-C).mass() = " << AplusBminusC.mass() << endl;
  cout << "(A+B-C).transverse() = " << AplusBminusC.transverse() << endl;

  LorentzTransverseVector AminusB = A-B;
  cout << "A-B = " << AminusB << endl;
  LorentzTransverseVector BminusA = B-A;
  cout << "B-A = " << BminusA << endl;

  double AdotB = A.dot(B);
  cout << "A.B = " << AdotB << endl;
  cout << "A.A = " << A.dot(A) << endl;
  cout << "A^2 + B^2 + 2 A.B = " <<  ( A.dot(A) + B.dot(B) + 2 * A.dot(B) ) << std::endl;
  cout << "sqrt((A+B)^2) = " << sqrt((A+B).dot(A+B)) << endl;

  double mInvisible = 150.*GeV;
  LorentzTransverseVector v1(TwoVector(1000.*GeV, 2200.*GeV), 1700.*GeV);
  LorentzTransverseVector v2(TwoVector(2000.*GeV, 1600.*GeV), 1700.*GeV);
  TwoVector ptmiss(1300.*GeV, 1100.*GeV);

  cout << "check equality \n"
       << LorentzTransverseVector(5.,3.,0.)
       << endl
       << LorentzTransverseVector(TwoVector(3.,0.),4.)
       << endl;

  for (int mini=1; mini<=2; ++mini) {
    for (int i=1; i<=2; ++i) {
      
      Mt2Minimiser * minimizer=0;
      string name;
      if (mini==1) {
	//minimizer = new Mt2MinimiserGsl();
	name = "Gsl";
      } else {
	//minimizer = new Mt2MinimiserNag();
	name = "Nag";
      }
      //assert(minimizer);

      const double rootSMax = i==1 ? 14000.*TeV : 14.*TeV;
      
      cout << "Using " <<name<<" attempt with rootSMax = " << rootSMax << endl;
      if (i==2) {
	cout << "\n\n\nExpect next mT2 evaluation to fail as configuration is inconsistent!\n\n\n" << endl;
      }
      
      //Mt2Calculator calc(minimizer, rootSMax);
      SUSYPhys_Mt2_222_Calculator calc;

      calc.setDebug(true);
      
      try{
	double mt2 = calc.mt2_222(v1.transverse(), v2.transverse(), ptmiss, mInvisible);
	cout << "M_T2 = " << mt2 << " " << endl;
      } catch (exception& e){
	cout << "M_T2 failed due to exception: [" << e.what() << "]" << endl;
      }
    }
  }
  cout << "END" << endl;
}
