#include <iostream>
#include "Mt2/Mt2Calculator.h"
#include "Mt2/Basic_Mt2_332_Calculator.h"
#include "Mt2/SUSYPhys_Mt2_222_Calculator.h"
#include "Mt2/Mt2MinimiserNag.h"
#include "Mt2/Mt2MinimiserGsl.h"
#include "Mt2/Mt2Units.h"
#include "Summers.h"
#include "../examples/ExampleEvent.h"

//const double MeV=0.001; 
//const double GeV=1000*MeV;
//const double TeV=1000*GeV;
// so unformatted output will be in GeV

int main(){
  using namespace Mt2;
  using namespace std;

  ExampleEvent exampleEvent;

  const double rootS = exampleEvent.rootS();
  const double mInvisible = exampleEvent.invis_mass();

  const Mt2::LorentzVector exampleA = exampleEvent.p_Vis_A();
  const Mt2::LorentzVector exampleB = exampleEvent.p_Vis_B();
  const Mt2::LorentzVector exampleG = exampleEvent.p_Vis_Other();

  // copying the numbers from investigation.f:

  const Mt2::LorentzVector exampleBigSigma = exampleA+exampleB+exampleG;

  const TwoVector v1 = exampleEvent.pT_Vis_A();
  const TwoVector v2 = exampleEvent.pT_Vis_B();
  const TwoVector ptmiss = exampleEvent.pT_Miss();

  cout << " v1 = " << v1 
       << "\n v2 = " << v2 
       << "\n ptmiss = " << ptmiss << endl;

  double summers = mt2Summers(v1,v2,ptmiss,mInvisible);
  cout  << "Summers = " << summers << endl;
  
  const LorentzTransverseVector alpha = exampleEvent.ltv_Vis_A();
  const LorentzTransverseVector beta  = exampleEvent.ltv_Vis_B();

  double lester4 = mt2Lester4(alpha,beta,
			      ptmiss,mInvisible);
  cout  << "Lester4 = " << lester4 << endl;

  double lester3 = mt2Lester3(v1,v2,
                              sqrt(exampleA.m2()), // This method assumes that the masses of both visible particles are the same - even if they are not!
			      ptmiss,mInvisible);
  cout  << "Lester3 = " << lester3 << endl;

  double lester2 = mt2Lester2(alpha,beta,
			      ptmiss,mInvisible);
  cout  << "Lester2 = " << lester2 << endl;

#ifdef IF_USE_SUSPECT
  // NAG alternative:

  Mt2Calculator calculatorNag (new Mt2MinimiserNag(), 14.*TeV);
  calculatorNag.getMinimiser()->setTolerance(1.E-10);
  calculatorNag.setDebug(true);

  const double resultNag = calculatorNag.mt2(alpha, beta,
				       ptmiss, mInvisible);

  // GSL alternative:

  Mt2Calculator calculatorGsl (new Mt2MinimiserGsl(), 14.*TeV);
  calculatorGsl.getMinimiser()->setTolerance(1.E-4);
  calculatorGsl.setDebug(true);

  const double resultGsl = calculatorGsl.mt2(alpha, beta,
				       ptmiss, mInvisible);
#endif

  // SUSYPhys + Minuit alternative
  std::cout << "Calculating with SUSYPhys + Minuit alternative" << std::endl;
  SUSYPhys_Mt2_222_Calculator calcSP;
  calcSP.setDebug(true);
  std::cout <<alpha << " " << beta<<" " <<ptmiss << " " << mInvisible << std::endl;
  const double resultSUSYPhysMinuit = calcSP.mt2_222(v1, v2,
						     ptmiss, mInvisible);

  // Basic + Minuit(Simplex) alternative
  std::cout << "Calculating with Basic + Minuit(Simplex) alternative" << std::endl;
  Basic_Mt2_332_Calculator calcB;
  calcB.setDebug(true);


  const double resultBasicMinuit = calcB.mt2_332(alpha, beta,
						ptmiss, mInvisible);
  
  // Basic_4441 alternative
  std::cout << "Calculating with Basic_4441 alternative" << std::endl;
  Basic_Mt2_4441_Calculator calc4441;
  calc4441.setDebug(true);

  const double result4441 
    = calc4441.mt2_4441(exampleA, // visA
			exampleB, // visB
			exampleG, // otherVis
			rootS,
			mInvisible 
			);
  // Summary:

  //cout  << "ResultNag = " << resultNag /*<< " WARNING: THIS RESULT GOT LUCKY.  Fix both #warnings in code before trusting the code in general!"*/ << endl;
  //cout  << "ResultGsl = " << resultGsl /*<< " WARNING: THIS RESULT GOT LUCKY.  Fix both #warnings in code before trusting the code in general!"*/ << endl;
  cout  << "ANS 1 " << calcSP.algorithmName() << "\t = " << resultSUSYPhysMinuit << " GeV" << endl;
  cout  << "ANS 2 " << calcB.algorithmName() << "\t = " << resultBasicMinuit << " GeV" << endl;
  cout  << "ANS 3 " << calc4441.algorithmName() << "\t = " << result4441 << " GeV" << endl;
  cout  << "ANS 4 Summers       \t = " << summers << " GeV" << endl;
  cout  << "ANS 5 Lester4       \t = " << lester4 << " GeV" << endl;
  cout  << "ANS 5aLester3       \t = " << lester3 << " GeV" << endl;
  cout  << "ANS 6 Lester2       \t = " << lester2 << " GeV" << endl;

#ifdef IF_USE_SUSPECT
  cout << "calcNag:" << calculatorNag.calls()
       <<  " " << calculatorNag.evaluations()
       <<  " " << calculatorNag.unbalancedEvaluations() << endl;
  cout << "calcGsl:" << calculatorGsl.calls()
       <<  " " << calculatorGsl.evaluations()
       <<  " " << calculatorGsl.unbalancedEvaluations() << endl;
#endif
}
