
/*

   This is an example ROOT macro that demonstrates
   how to evalutate MT2 in ROOT semi-interactively.
   It is important to realise that you MUST use the 
   ACLIC compiler if you want to use ROOT as CINT is
   not good enough on its own.  Follow the instructions
   below which have been tested on root v 5.15/08.

   FIRST:
        run "make" in the current dir to build
        "src/libMt2.so".

   THEN: run me frominside ROOT like this:

        gSystem->Load("libMinuit2.so");
        gSystem->Load("src/libMt2.so");
        .L mt2_Basic_332_ROOT_example.C+
        mt2_Basic_332_ROOT_example();
  
*/

#include "Mt2/Basic_Mt2_332_Calculator.h"

void mt2_Basic_332_ROOT_example() {

  Mt2::Basic_Mt2_332_Calculator calc;

  for (unsigned int event=0; event<10; ++event) {
    double m_vis_A_mass=100;
    double m_vis_B_mass=150;
    double m_invis_mass=100;
    
    Mt2::LorentzTransverseVector vis_A(Mt2::TwoVector( 410,   20), m_vis_A_mass);
    Mt2::LorentzTransverseVector vis_B(Mt2::TwoVector(-210, -300), m_vis_B_mass);
    Mt2::TwoVector pT_Miss = -((vis_A+vis_B).transverse());

    const double mT2 = calc.mt2_332(vis_A,
				    vis_B,
				    pT_Miss,
				    m_invis_mass);

    std::cout << mT2 << std::endl;
  }

}
