
/*

   This is an example ROOT macro that demonstrates
   how to evalutate MTGEN in ROOT semi-interactively.
   It is important to realise that you MUST use the 
   ACLIC compiler if you want to use ROOT as CINT is
   not good enough on its own.  Follow the instructions
   below which have been tested on root v 5.15/08.

   EITHER:
      run me like this:

        .L src/Mt2Vectors.cpp+
        .L src/Mt2Calculator.cpp+
        .L src/Analytic_Mt2_330_Calculator.cpp+
        .L mtgen_ROOT_example.C+
        mtgen_ROOT_example();

  OR:
     run "make" in the current dir to build
     "src/libMt2Standalone.so" and then
     run me from inside ROOT like this:

        gSystem->Load("src/libMt2Standalone.so");
        .L mtgen_ROOT_example.C+
        mtgen_ROOT_example();
  
*/

#include "Mt2/Basic_MtGen_330_Calculator.h"
#include "Mt2/Analytic_Mt2_330_Calculator.h"

void mtgen_ROOT_example() {

  // Create an Mt2_330 calculator which we will ask the MtGen calculator to use later:
  Mt2::Analytic_Mt2_330_Calculator mt2_330_Calculator;

  // Ask MtGen to use the above Mt2_330_Calculator:
  Mt2::Basic_MtGen_330_Calculator mtGenCalculator(mt2_330_Calculator);

  for (unsigned int event=0; event<10; ++event) {
    double m_vis_A_mass=100;
    double m_vis_B_mass=150;
    double m_invis_mass=100;
    
    Mt2::LorentzTransverseVector vis_A(Mt2::TwoVector( 410,   20), m_vis_A_mass);
    Mt2::LorentzTransverseVector vis_B(Mt2::TwoVector(-210, -300), m_vis_B_mass);
    
    std::vector<Mt2::LorentzTransverseVector> interestingParticles;
    interestingParticles.push_back(vis_A);
    interestingParticles.push_back(vis_B);
    
    const double mT_Gen = mtGenCalculator.mtGen_330(interestingParticles, m_invis_mass);

    std::cout << mT_Gen << std::endl;
  }

}
