#include "Mt2/Mt2MinimiserNag.h"
#include "Mt2/Mt2Calculator.h"
#include <nag.h>
#include <nage04.h>

namespace Mt2 {

Mt2MinimiserNag::Mt2MinimiserNag() : 
  m_tolerance (1E-8),
  m_max_fun (100)
{
}

Mt2MinimiserNag::~Mt2MinimiserNag()
{
}

double Mt2MinimiserNag::minimise(const Mt2Calculator& calc){
  double e1 = m_tolerance;
  double e2 = m_tolerance;
  double par_min = 0.;
  double par_max = calc.rootSMax-calc.rootSMin;
  if (par_max<par_min) {
    throw EventIsIncompatibleWithRootS_Exception();
  }
  if (par_min+e2 >= par_max) {
    throw EventIsAlmostIncompatibleWithRootS_Exception();
  }

  Nag_Comm comm;
  comm.p = (void*) &calc;
  double x=0.;
  double value=0.;
  NagError nagError;
  INIT_FAIL ( nagError ) ;
#ifndef NDEBUG
  SET_FAIL( nagError ); // prints error message to stdout
#endif
  nag_opt_one_var_no_deriv (static_funct,
			    e1,  e2,  &par_min,  &par_max, 
			    m_max_fun,  &x,  &value, &comm, &nagError);
  
  if (nagError.code != NE_NOERROR ) {
    throw ProgramWasNotCleverEnoughException("The NAG minimiser refused to calculate mT2 for us.");
  }
  return value;
}

void Mt2MinimiserNag::static_funct(double x, double* f, Nag_Comm* comm){
  Mt2Calculator *calc = (Mt2Calculator*) comm->p;
  *f = calc->trialMt2(x);
}

void Mt2MinimiserNag::setTolerance(double t){
  m_tolerance = t;
}

void Mt2MinimiserNag::setMaxEvaluations(int i){
  m_max_fun = i;
}

}
